Package: polyglotr
Title: Translate Text
Version: 1.7.4
Authors@R: 
    person("Tomer", "Iwan", , "iwantomer@hotmail.com", role = c("aut", "cre", "cph"))
Description: Provide easy methods to translate pieces of text.
  Functions send requests to translation services online.
License: MIT + file LICENSE
URL: https://github.com/Tomeriko96/polyglotr/,
        https://tomeriko96.github.io/polyglotr/
BugReports: https://github.com/Tomeriko96/polyglotr/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: dplyr, httr, jsonlite, magrittr, purrr, RCurl, rlang, rvest,
        stringr, tibble, urltools
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), text2vec, shiny,
        shinydashboard, shinyjs, DT
Config/testthat/edition: 3
VignetteBuilder: knitr
Depends: R (>= 3.5.0)
LazyData: true
NeedsCompilation: no
Packaged: 2026-06-08 07:27:44 UTC; dev
Author: Tomer Iwan [aut, cre, cph]
Maintainer: Tomer Iwan <iwantomer@hotmail.com>
Repository: CRAN
Date/Publication: 2026-06-08 08:20:02 UTC
Built: R 4.5.2; ; 2026-06-08 12:48:13 UTC; unix
