# Generated by roxygen2: do not edit by hand

export("%>%")
export(apertium_get_language_pairs)
export(apertium_translate)
export(create_translation_table)
export(create_transliteration_table)
export(google_get_supported_languages)
export(google_is_valid_language_code)
export(google_translate)
export(google_translate_long_text)
export(google_transliterate)
export(language_detect)
export(launch_polyglotr_app)
export(microsoft_supported_languages)
export(mymemory_translate)
export(pons_dictionaries)
export(pons_translate)
export(qcri_get_domains)
export(qcri_get_language_pairs)
export(qcri_translate_text)
export(translate_file)
export(translate_to_morse)
export(translate_to_morse_audio)
export(wikimedia_detect_language)
export(wikipedia_get_language_names)
export(wmcloud_translate)
importFrom(magrittr,"%>%")
importFrom(rlang,":=")
